@@ -2,12 +2,15 @@
 #include "decl.h"
 #include "rcpp_matrix.h"
 #include "rcpp_constraint.h"
-#include <adelie_core/state/state_gaussian_pin_cov.hpp>
-#include <adelie_core/state/state_gaussian_cov.hpp>
-#include <adelie_core/state/state_gaussian_naive.hpp>
-#include <adelie_core/state/state_glm_naive.hpp>
-#include <adelie_core/state/state_multigaussian_naive.hpp>
-#include <adelie_core/state/state_multiglm_naive.hpp>
+#include <adelie_core/state/state_gaussian_pin_base.ipp>
+#include <adelie_core/state/state_gaussian_pin_cov.ipp>
+#include <adelie_core/state/state_gaussian_pin_naive.ipp>
+#include <adelie_core/state/state_base.ipp>
+#include <adelie_core/state/state_gaussian_cov.ipp>
+#include <adelie_core/state/state_gaussian_naive.ipp>
+#include <adelie_core/state/state_glm_naive.ipp>
+#include <adelie_core/state/state_multigaussian_naive.ipp>
+#include <adelie_core/state/state_multiglm_naive.ipp>
 
 using state_base_64_t = ad::state::StateBase<
     constraint_base_64_t, double, int, int, int
